******************************************************
******** Motecarlo Study: experiment 2, 50% missing **
******************************************************

/* netpath to data */
global netdtapath "/Volumes/Archive/Dropbox/Stata/MxMeeting/SUGMX/Mx21/code/montecarlo_exp2"


/* Local path to data */
global localdtapath "/Volumes/Archive/Dropbox/Stata/MxMeeting/SUGMX/Mx21/code/montecarlo_exp2"


/* Local log path */
global locallogpath "/Volumes/Archive/Dropbox/Stata/MxMeeting/SUGMX/Mx21/code/montecarlo_exp2"

/* 2SLS no selection estimation program */

capture program drop ivnosel_estimator
program define ivnosel_estimator, eclass 
 version 12.1
 syntax varlist [if] [in], selvar(varname) ealvar(varname) selzvars(varlist) ealzvars(varlist) yzvars(varlist)
 gettoken y xvars:varlist, parse("")
 marksample touse
 markout `touse' `xvars' `selvar' `ealvar' `selzvars' `ealzvars' 
 tempname gradient mV glen nrlen conv rank ealpred emppred convg
 tempvar bvprobit_selpred bvprobit_ealpred bvprobit_rho ealpred prsel premp
 /* sum vars */
 sum `y' `selvar' `ealvar'
 * clean IMR and ealpred vars
 replace IMR = .
 replace GIMRs = .
 replace GIMRt = .
 replace IGIMRs = .
 replace IGIMRt = .
 replace ealpred = .
 * step 1: wage equation
 ivregress 2sls `y' `xvars' `yzvars' (`ealvar' = `ealzvars') if `y'!=.
 scalar `rank' = e(rank)
 if (`rank'<7) {
  scalar `convg' = 0
 }
 else{
  scalar `convg' = 1
 }
 /* return estimated paramters */
 mat b = e(b)
 mat V = e(V)
 ereturn post b V
 ereturn scalar converged = `convg'
 ereturn scalar rank = `rank'
end 
 
 
/* naive 2SLS estimation program */

capture program drop naive2SE_estimator
program define naive2SE_estimator, eclass 
 version 12.1
 syntax varlist [if] [in], selvar(varname) ealvar(varname) selzvars(varlist) ealzvars(varlist) yzvars(varlist)
 gettoken y xvars:varlist, parse("")
 marksample touse
 markout `touse' `xvars' `selvar' `ealvar' `selzvars' `ealzvars' 
 tempname gradient mV glen nrlen conv rankprobit rankheckman rank ealpred emppred convg
 tempvar bvprobit_selpred bvprobit_ealpred bvprobit_rho ealpred prsel premp
 /* sum vars */
 sum `y' `selvar' `ealvar'
 * clean IMR and ealpred vars
 replace IMR = .
 replace GIMRs = .
 replace GIMRt = .
 replace IGIMRs = .
 replace IGIMRt = .
 replace ealpred = .
 * step 1: EAL equation
 probit `ealvar' `xvars' `ealzvars'
 predict `ealpred', pr
 replace ealpred = `ealpred'
 scalar `rankprobit' = e(rank)
 if (`rankprobit'<6) {
  scalar `convg' = 0
 }
 else {
  scalar `convg' = 1
 }
 * step 2: heckman regression for wage
 heckman `y' ealpred `xvars' `yzvars', select(`selvar' = `xvars' `selzvars') twostep 
 scalar `rankheckman' = e(rank)
 if (`convg' == 0 | `rankheckman'<14) {
  scalar `convg' = 0
 }
 scalar `rank' = `rankheckman'
 /* return estimated paramters */
 mat b = e(b)
 mat V = e(V)
 ereturn post b V
 ereturn scalar converged = `convg'
 ereturn scalar rank = `rank'
end 
 
/* Reize2SE estimation program */

capture program drop Reize2SE_estimator
program define Reize2SE_estimator, eclass 
 version 12.1
 syntax varlist [if] [in], selvar(varname) ealvar(varname) selzvars(varlist) ealzvars(varlist) yzvars(varlist)
 gettoken y xvars:varlist, parse("")
 marksample touse
 markout `touse' `xvars' `selvar' `ealvar' `selzvars' `ealzvars' 
 tempname gradient mV glen nrlen conv rankols rankbiprobit rankReize2SE rank ealpred emppred convg
 tempvar bvprobit_selpred bvprobit_ealpred bvprobit_rho ealpred prsel premp
 /* sum vars */
 sum `y' `selvar' `ealvar'
 * clean IMR and ealpred vars
 replace IMR = .
 replace GIMRs = .
 replace GIMRt = .
 replace ealpred = .
 * step 1: (seemingly unrelated) biprobit of EAL and Selection	
 biprobit (`ealvar' = `xvars' `ealzvars') (`selvar' = `ealvar' `xvars' `selzvars') ,  iterate(30) 
 predict `bvprobit_ealpred', xb1 
 predict `bvprobit_selpred', xb2 
 scalar `bvprobit_rho'=e(rho) 
 #delimit ;
  replace GIMRs = `ealvar'*normalden(`bvprobit_selpred')*(normal((`bvprobit_ealpred'-`bvprobit_rho'*`bvprobit_selpred')/sqrt(1-`bvprobit_rho'^2))/binormal(`bvprobit_ealpred',`bvprobit_selpred',`bvprobit_rho'))+
   (1-`ealvar')*normalden(`bvprobit_selpred')*(normal((-`bvprobit_ealpred'-`bvprobit_rho'*`bvprobit_selpred')/sqrt(1-`bvprobit_rho'^2))/binormal(-`bvprobit_ealpred',`bvprobit_selpred',-`bvprobit_rho'));
  replace GIMRt = `ealvar'*normalden(`bvprobit_ealpred')*(normal((`bvprobit_selpred'-`bvprobit_rho'*`bvprobit_ealpred')/sqrt(1-`bvprobit_rho'^2))/binormal(`bvprobit_ealpred',`bvprobit_selpred',`bvprobit_rho'))-
   (1-`ealvar')*normalden(-`bvprobit_ealpred')*(normal((`bvprobit_selpred'+`bvprobit_rho'*`bvprobit_ealpred')/sqrt(1-`bvprobit_rho'^2))/binormal(-`bvprobit_ealpred',`bvprobit_selpred',-`bvprobit_rho'));
 #delimit cr
 scalar `convg' = e(converged)
 scalar `rankbiprobit' = e(rank)
 if  (`convg'==0 | `rankbiprobit'<14) {
   scalar `convg' = 0
 }
 * step 2: wage equation
 regress `y' `ealvar' `xvars' `yzvars' GIMRs GIMRt if `y'!=.
 scalar `rankReize2SE' = e(rank)
 if (`rankReize2SE'<9) {
  scalar `convg' = 0
 }
 scalar `rank' = `rankReize2SE'
 /* return estimated paramters */
 mat b = e(b)
 mat V = e(V)
 ereturn post b V
 ereturn scalar converged = `convg'
 ereturn scalar rank = `rank'
end 


/* Wooldridge alternative */

capture program drop Wool3SE_estimator
program define Wool3SE_estimator, eclass 
 version 12.1
 syntax varlist [if] [in], selvar(varname) ealvar(varname) selzvars(varlist) ealzvars(varlist) yzvars(varlist)
 gettoken y xvars:varlist, parse("")
 marksample touse
 markout `touse' `xvars' `selvar' `ealvar' `selzvars' `ealzvars' 
 tempname gradient mV glen nrlen conv rankols rankbiprobit rankwool rank ealpred emppred convg
 tempvar bvprobit_selpred bvprobit_ealpred bvprobit_rho ealpred probit_selpred prsel premp
 /* sum vars */
 sum `y' `selvar' `ealvar'
 * clean IMR and ealpred vars
 replace IMR = .
 * step 1: EAL equation
 reg `ealvar' `xvars' `ealzvars'
 predict `ealpred', xb
 replace ealpred = `ealpred'
 scalar `rankols' = e(rank)
 if (`rankols'<6) {
  scalar `convg' = 0
 }
 * step 2: probit for Selection	
 probit `selvar' `xvars' `ealzvars' `selzvars'
 predict `probit_selpred', xb 
 replace IMR = normalden(`probit_selpred')/normprob(`probit_selpred')
 scalar `convg' = e(converged)
 * step 3: wage equation
 ivregress 2sls `y' `xvars' `yzvars' (`ealvar' = ealpred)  IMR if `y'!=.
 scalar `rankwool' = e(rank)
 if (`rankwool'<8) {
  scalar `convg' = 0
 }
 scalar `rank' = `rankwool'
 /* return estimated paramters */
 mat b = e(b)
 mat V = e(V)
 ereturn post b V
 ereturn scalar converged = `convg'
 ereturn scalar rank = `rank'
end 
  
/* set more off */
clear
clear matrix
clear mata
set more off
 
/* MODEL GLOBALS */
** exogenous vars
global xvars "x1 x2 d1 d2"
**OLS regression
global ols_xvars "treat x1 x2 d1 d2"
**ivnosel
global ivnosel_xvars "treat x1 x2 d1 d2"
**TSLS 
global tsls_xvars "treat x1 x2 d1 d2"
** endogenous treatment xvars
global treat_xvars "x1 x2 d1 d2"
** Endogenous treatment instrument
global treat_zvars "ztreat"
** selection xvars
global sel_xvars "treat x1 x2 d1 d2"
** Sample selection instrument
global sel_zvars "zsel"
** main response instrument 
global y_zvar "zyvar"
** Sample size
global ssize = 1000
** Monte Carlo repetitions
global rep=5
** Seed
global seed=8873363

/* SET PUPOLATION PARAMETERS*/

/* temporal names */
#delimit ;
tempname 
nrun nfail psel nsel ptreat ntreat
mu_y_t mu_y_x1 mu_y_x2 mu_y_d1 mu_y_d2 mu_y_zyvar mu_y_cons 
mu_t_x1 mu_t_x2 mu_t_d1 mu_t_d2 mu_t_ztreat mu_t_cons 
mu_s_t mu_s_x1 mu_s_x2 mu_s_d1 mu_s_d2 mu_s_zsel mu_s_cons 
mu_rho_yt mu_rho_ys mu_rho_ts 
mu_sd_y mu_sd_t mu_sd_s
noise2sig_y noise2sig_t noise2sig_s psel nsel junk nplus;
#delimit cr

/* set matrix PP */
local udistr "Normal"
mat PP=J(1,32,0) 
local n=$ssize
mat PP[1,1]=`n'
local rep=$rep
mat PP[1,2]=`rep'
**standard deviations
scalar `mu_sd_y'=0.79
mat PP[1,3]=`mu_sd_y'
scalar `mu_sd_t'=1
mat PP[1,4]=`mu_sd_t'
scalar `mu_sd_s'=1
mat PP[1,5]=`mu_sd_s'
mat SD=(`mu_sd_y',`mu_sd_t',`mu_sd_s')
mat list SD
**correlations
scalar `mu_rho_yt'=-0.2
mat PP[1,6]=`mu_rho_yt'
scalar `mu_rho_ys'=0.8
mat PP[1,7]=`mu_rho_ys'
scalar `mu_rho_ts'=-0.2
mat PP[1,8]=`mu_rho_ts'
mat CORR = J(3,3,1)
mat CORR[1,2]=`mu_rho_yt'
mat CORR[2,1]=`mu_rho_yt'
mat CORR[1,3]=`mu_rho_ys'
mat CORR[3,1]=`mu_rho_ys'
mat CORR[2,3]=`mu_rho_ts'
mat CORR[3,2]=`mu_rho_ts'
** endogenous treatment
scalar `mu_t_x1'=-0.58
mat PP[1,9]=`mu_t_x1'
scalar `mu_t_x2'=0.58
mat PP[1,10]=`mu_t_x2'
scalar `mu_t_d1'=-0.58
mat PP[1,11]=`mu_t_d1'
scalar `mu_t_d2'=0.58
mat PP[1,12]=`mu_t_d2'
scalar `mu_t_ztreat'=1.8
mat PP[1,13]=`mu_t_ztreat'
scalar `mu_t_cons'=0.02
mat PP[1,14]=`mu_t_cons'
** main response y
scalar `mu_y_t'= 1
mat PP[1,15]=`mu_y_t'
scalar `mu_y_x1'= 1
mat PP[1,16]=`mu_y_x1'
scalar `mu_y_x2'=-1
mat PP[1,17]=`mu_y_x2'
scalar `mu_y_d1'=1
mat PP[1,18]=`mu_y_d1'
scalar `mu_y_d2'=-1
mat PP[1,19]=`mu_y_d2'
scalar `mu_y_zyvar'=1
mat PP[1,20]=`mu_y_zyvar'
scalar `mu_y_cons'=1
mat PP[1,21]=`mu_y_cons'
** selection dummy s
scalar `mu_s_t'= 1.2
mat PP[1,22]=`mu_s_t'
scalar `mu_s_x1'= -0.12
mat PP[1,23]=`mu_s_x1'
scalar `mu_s_x2'=0.12
mat PP[1,24]=`mu_s_x2'
scalar `mu_s_d1'=-0.12
mat PP[1,25]=`mu_s_d1'
scalar `mu_s_d2'=0.12
mat PP[1,26]=`mu_s_d2'
scalar `mu_s_zsel'=-1.75
mat PP[1,27]=`mu_s_zsel'
scalar `mu_s_cons'=-0.6
mat PP[1,28]=`mu_s_cons'
**signal/noise ratios
#delimit ;
scalar `junk' = `mu_t_x1'+`mu_t_x2'+`mu_t_d1'+`mu_t_d1'+`mu_t_ztreat'+`mu_t_cons';
scalar `noise2sig_y' = (`mu_sd_y'^2) / (`mu_y_t'*normal(`junk')*(1-normal(`junk'))
  +`mu_y_x1'^2+ `mu_y_x2'^2+`mu_y_d1'*0.25+`mu_y_d2'*0.25+`mu_y_zyvar'*0.25) ;
mat PP[1,29] = `noise2sig_y';
scalar `noise2sig_t' = (`mu_sd_t'^2)/(`mu_t_x1'^2+ `mu_t_x2'^2+`mu_t_d1'*0.25+`mu_t_d2'*0.25
 +`mu_t_ztreat'^2);
mat PP[1,30] = `noise2sig_t';
scalar `noise2sig_s' = (`mu_sd_s'^2) / (`mu_s_t'*normal(`junk')*(1-normal(`junk'))
 +`mu_s_x1'^2+`mu_s_x2'^2+`mu_s_d1'*0.25+`mu_s_d2'*0.25
 + `mu_s_zsel'^2)  ;
mat PP[1,31] = `noise2sig_s';
#delimit cr
/*seed */
local seed = $seed
mat PP[1,32] = `seed'
/* list population parameters */
mat list PP
/* list correlation matrix */
mat list CORR
/* list standard deviations */
mat list SD 
/* Prepare Post file */
tempname pname
#delimit ;
postfile `pname'
 double(
 nrun nfail psel nsel ptreat ntreat
 /* OLS  */
 ols_treat ols_treat_se
 ols_x1  ols_x1_se
 ols_x2  ols_x2_se
 ols_d1  ols_d1_se
 ols_d2  ols_d2_se
 ols_zyvar ols_zyvar_se
 /* IV no selection */
 ivnosel_treat ivnosel_treat_se
 ivnosel_x1  ivnosel_x1_se
 ivnosel_x2  ivnosel_x2_se
 ivnosel_d1  ivnosel_d1_se
 ivnosel_d2  ivnosel_d2_se
 ivnosel_zyvar ivnosel_zyvar_se
 /* naive2SE */
 naive2SE_treat naive2SE_treat_se
 naive2SE_x1  naive2SE_x1_se
 naive2SE_x2  naive2SE_x2_se
 naive2SE_d1  naive2SE_d1_se
 naive2SE_d2  naive2SE_d2_se
 naive2SE_zyvar naive2SE_zyvar_se
 /* Reizet2SE */
 Reize2SE_treat Reize2SE_treat_se
 Reize2SE_x1  Reize2SE_x1_se
 Reize2SE_x2  Reize2SE_x2_se
 Reize2SE_d1  Reize2SE_d1_se
 Reize2SE_d2  Reize2SE_d2_se
 Reize2SE_zyvar Reize2SE_zyvar_se
 /* Wooldridge */
 Wool3SE_treat Wool3SE_treat_se
 Wool3SE_x1  Wool3SE_x1_se
 Wool3SE_x2  Wool3SE_x2_se
 Wool3SE_d1  Wool3SE_d1_se
 Wool3SE_d2  Wool3SE_d2_se
 Wool3SE_zyvar Wool3SE_zyvar_se
)
using "$localdtapath/Montecarlo_exp2.dta", every(1) replace;
#delimit cr
/* set seed */
set seed `seed'
/* set locals */
local i = 1
local flag = 0
local nfail = 0
local nrun = 0
while `i'<=`rep' {
 **SIMULATE THE DATA
 clear
 set obs `n' 
 gen id = _n
 gen double x1=rnormal()
 gen double x2=rnormal()
 gen double ztreat = rnormal()
 gen double zsel = rnormal()
 gen double zyvar = rnormal()
 gen d1 = rnormal()>0
 gen d2 = rnormal()>0
 gen double y = .
 gen double treatstar = .
 gen double selstar = .
 gen treat = .
 gen sel = .
 gen double try = .
 gen double trtreatstar = .
 gen double trtreat = .
 gen double trselstar = . 
 gen double trsel = .
 gen double trpsel = .
 gen double IMR = .
 gen double GIMRs = .
 gen double GIMRt = .
 gen double IGIMRs = .
 gen double IGIMRt = .
 gen double ealpred = .
 /* calculate true y, treat, s */
 #delimit ;
 replace trtreatstar = `mu_t_cons' 
  + `mu_t_x1'*x1 + `mu_t_x2'*x2 + `mu_t_d1'*d1 + `mu_t_d2'*d2;
 replace trselstar = `mu_s_cons' + `mu_s_t'*0.11
  + `mu_s_x1'*x1 + `mu_s_x2'*x2 + `mu_s_d1'*d1 + `mu_s_d2'*d2;
 replace try = `mu_y_cons' + `mu_y_t'*0.11
  + `mu_y_x1'*x1 + `mu_y_x2'*x2 + `mu_y_d1'*d1 + `mu_y_d2'*d2
  + `mu_y_zyvar'*zyvar;
 #delimit cr
 /* calulate true selection probability */
 #delimit ;
 replace trpsel = normal(trselstar);
 di trpsel;
 #delimit cr
 /* generate random draws from multivariate normal */
 drawnorm ey et es, corr(CORR) mean(0,0,0) sds(SD)
 corr ey et es
 /* generate simulated responses */
 #delimit ;
 replace treatstar = `mu_t_cons' 
  + `mu_t_x1'*x1 + `mu_t_x2'*x2 + `mu_t_d1'*d1 + `mu_t_d2'*d2
  + `mu_t_ztreat'*ztreat + et;
 replace treat =  treatstar>0;
 replace selstar = `mu_s_cons' + `mu_s_t'*treat
  + `mu_s_x1'*x1 + `mu_s_x2'*x2 + `mu_s_d1'*d1 
  + `mu_s_d2'*d2 + `mu_s_zsel'*zsel + es;
 replace sel =  selstar>0;
 replace y = `mu_y_cons' + `mu_y_t'*treat
  + `mu_y_x1'*x1 + `mu_y_x2'*x2 + `mu_y_d1'*d1 
  + `mu_y_d2'*d2 + `mu_y_zyvar'*zyvar + ey;
 #delimit cr
 /* Make y missing following selection rule */
 replace y=. if sel==0
 /* calculate probability of selection */
 su sel
 scalar `psel' = r(mean)
 scalar `nsel' = r(sum)
 su treat 
 scalar `ptreat' = r(mean)
 scalar `ntreat' = r(sum)
 /* save simulated data */
 save "$localdtapath/sdat/exp2/exp2_`i'.dta", replace
/* Fit OLS model */
 regress y treat $xvars zyvar
 mat b=e(b)
 mat V=e(V)
 tempname rank
 scalar `rank' = e(rank)
 if (`rank'<7) {
  di "Matrix of X is not of full rank [i.e., Rank(X)<K]"
  di "Random draw discarded and new draw taken"
  if (`flag'==1) {
   local i = `i' - 1
   local flag = 0
   local nfail = `nfail' + 1
  }
  local nrun = `nrun' + 1
  continue
 }
 local ols_treat = b[1,1]
 local ols_x1 =  b[1,2]
 local ols_x2 =  b[1,3]
 local ols_d1 =  b[1,4]
 local ols_d2 =  b[1,5]
 local ols_zyvar =  b[1,6]
 local ols_treat_se =  sqrt(V[1,1])
 local ols_x1_se =  sqrt(V[2,2])
 local ols_x2_se =  sqrt(V[3,3])
 local ols_d1_se =  sqrt(V[4,4])
 local ols_d2_se =  sqrt(V[5,5])
 local ols_zyvar_se =  sqrt(V[6,6])
 /* Fit Naive 2SLS (ivnosel) */
 #delimit ;
  ivnosel_estimator y $xvars,  selvar(sel) ealvar(treat) 
   selzvars(zsel) ealzvars(ztreat) yzvars(zyvar);
 #delimit cr
 mat b=e(b)
 mat V=e(V)
 tempname rank
 scalar `rank' = e(rank)
 if (`rank'<7) {
  di "Matrix of X is not of full rank [i.e., Rank(X)<K]"
  di "Random draw discarded and new draw taken"
  if (`flag'==1) {
   local i = `i' - 1
   local flag = 0
   local nfail = `nfail' + 1
  }
  local nrun = `nrun' + 1
  continue
 }
 local ivnosel_treat = b[1,1]
 local ivnosel_x1 =  b[1,2]
 local ivnosel_x2 =  b[1,3]
 local ivnosel_d1 =  b[1,4]
 local ivnosel_d2 =  b[1,5]
 local ivnosel_zyvar = b[1,6]
 local ivnosel_treat_se =  sqrt(V[1,1])
 local ivnosel_x1_se =  sqrt(V[2,2])
 local ivnosel_x2_se =  sqrt(V[3,3])
 local ivnosel_d1_se =  sqrt(V[4,4])
 local ivnosel_d2_se =  sqrt(V[5,5])
 local ivnosel_zyvar_se  = sqrt(V[6,6])
 /* Fit naive2SE */
 #delimit ;
  bootstrap _b, reps(50) 
 nowarn reject(e(converged)!=1) double:
  naive2SE_estimator y $xvars,  selvar(sel) ealvar(treat) 
   selzvars(zsel) ealzvars(ztreat) yzvars(zyvar);
 #delimit cr
 mat b=e(b)
 mat V=e(V)
 tempname rank
 scalar `rank' = e(rank)
 if (`rank'<14) {
  di "Matrix of X is not of full rank [i.e., Rank(X)<K]"
  di "Random draw discarded and new draw taken"
  if (`flag'==1) {
   local i = `i' - 1
   local flag = 0
   local nfail = `nfail' + 1
  }
  local nrun = `nrun' + 1
  continue
 }
 local naive2SE_treat = b[1,1]
 local naive2SE_x1 =  b[1,2]
 local naive2SE_x2 =  b[1,3]
 local naive2SE_d1 =  b[1,4]
 local naive2SE_d2 =  b[1,5]
 local naive2SE_zyvar = b[1,6]
 local naive2SE_treat_se =  sqrt(V[1,1])
 local naive2SE_x1_se =  sqrt(V[2,2])
 local naive2SE_x2_se =  sqrt(V[3,3])
 local naive2SE_d1_se =  sqrt(V[4,4])
 local naive2SE_d2_se =  sqrt(V[5,5])
 local naive2SE_zyvar_se  = sqrt(V[6,6])
  /* Fit Reize2SE */
 #delimit ;
 bootstrap _b, reps(50) 
  nowarn reject(e(converged)!=1) double:
  Reize2SE_estimator y $xvars,  selvar(sel) ealvar(treat) 
   selzvars(zsel) ealzvars(ztreat) yzvars(zyvar);
 #delimit cr
 mat b=e(b)
 mat V=e(V)
 tempname rank
 scalar `rank' = e(rank)
 if (`rank'<9) {
  di "Matrix of X is not of full rank [i.e., Rank(X)<K]"
  di "Random draw discarded and new draw taken"
  if (`flag'==1) {
   local i = `i' - 1
   local flag = 0
   local nfail = `nfail' + 1
  }
  local nrun = `nrun' + 1
  continue
 }
 local Reize2SE_treat = b[1,1]
 local Reize2SE_x1 =  b[1,2]
 local Reize2SE_x2 =  b[1,3]
 local Reize2SE_d1 =  b[1,4]
 local Reize2SE_d2 =  b[1,5]
 local Reize2SE_zyvar = b[1,6]
 local Reize2SE_treat_se =  sqrt(V[1,1])
 local Reize2SE_x1_se =  sqrt(V[2,2])
 local Reize2SE_x2_se =  sqrt(V[3,3])
 local Reize2SE_d1_se =  sqrt(V[4,4])
 local Reize2SE_d2_se =  sqrt(V[5,5])
 local Reize2SE_zyvar_se  = sqrt(V[6,6])
 /* Fit Wooldridge */
 #delimit ;
  bootstrap _b, reps(50) 
 nowarn reject(e(converged)!=1) double:
  Wool3SE_estimator y $xvars,  selvar(sel) ealvar(treat) 
   selzvars(zsel) ealzvars(ztreat) yzvars(zyvar);
 #delimit cr
 mat b=e(b)
 mat V=e(V)
 scalar `rank' = e(rank)
 if (`rank'<8) {
  di "Matrix of X is not of full rank [i.e., Rank(X)<K]"
  di "Random draw discarded and new draw taken"
  if (`flag'==1) {
   local i = `i' - 1
   local flag = 0
   local nfail = `nfail' + 1
  }
  local nrun = `nrun' + 1
  continue
 }
 local Wool3SE_treat = b[1,1]
 local Wool3SE_x1 =  b[1,2]
 local Wool3SE_x2 =  b[1,3]
 local Wool3SE_d1 =  b[1,4]
 local Wool3SE_d2 =  b[1,5]
 local Wool3SE_zyvar =  b[1,6]
 local Wool3SE_treat_se =  sqrt(V[1,1])
 local Wool3SE_x1_se =  sqrt(V[2,2])
 local Wool3SE_x2_se =  sqrt(V[3,3])
 local Wool3SE_d1_se =  sqrt(V[4,4])
 local Wool3SE_d2_se =  sqrt(V[5,5])
 local Wool3SE_zyvar_se =  sqrt(V[6,6])
 /* Posting estimates*/
 #delimit ;
  post `pname'
 /* Convergence info */
 (`nrun')
 (`nfail')
 (`psel')
 (`nsel')
 (`ptreat')
 (`ntreat')
 /* OLS */
 (`ols_treat')
 (`ols_treat_se')
 (`ols_x1')
 (`ols_x1_se')
 (`ols_x2')
 (`ols_x2_se')
 (`ols_d1')
 (`ols_d1_se')
 (`ols_d2')
 (`ols_d2_se')
 (`ols_zyvar')
 (`ols_zyvar_se')
 /* ivnosel */
 (`ivnosel_treat')
 (`ivnosel_treat_se')
 (`ivnosel_x1')
 (`ivnosel_x1_se')
 (`ivnosel_x2')
 (`ivnosel_x2_se')
 (`ivnosel_d1')
 (`ivnosel_d1_se')
 (`ivnosel_d2')
 (`ivnosel_d2_se')
 (`ivnosel_zyvar')
 (`ivnosel_zyvar_se')
 /* Naive2SE */
 (`naive2SE_treat')
 (`naive2SE_treat_se')
 (`naive2SE_x1')
 (`naive2SE_x1_se')
 (`naive2SE_x2')
 (`naive2SE_x2_se')
 (`naive2SE_d1')
 (`naive2SE_d1_se')
 (`naive2SE_d2')
 (`naive2SE_d2_se')
 (`naive2SE_zyvar')
 (`naive2SE_zyvar_se')
 /* Reize2SE */
 (`Reize2SE_treat')
 (`Reize2SE_treat_se')
 (`Reize2SE_x1')
 (`Reize2SE_x1_se')
 (`Reize2SE_x2')
 (`Reize2SE_x2_se')
 (`Reize2SE_d1')
 (`Reize2SE_d1_se')
 (`Reize2SE_d2')
 (`Reize2SE_d2_se')
 (`Reize2SE_zyvar')
 (`Reize2SE_zyvar_se')
 /* Wool3SE */
 (`Wool3SE_treat')
 (`Wool3SE_treat_se')
 (`Wool3SE_x1')
 (`Wool3SE_x1_se')
 (`Wool3SE_x2')
 (`Wool3SE_x2_se')
 (`Wool3SE_d1')
 (`Wool3SE_d1_se')
 (`Wool3SE_d2')
 (`Wool3SE_d2_se')
 (`Wool3SE_zyvar')
 (`Wool3SE_zyvar_se');
 #delimit cr
 /* step */
 local i = `i' + 1
 local flag = 1
 local nrun = `nrun' + 1
}
di in gre _continue "."
postclose `pname'

/* Generate Monte Carlo Statistics */
qui use "$localdtapath/Montecarlo_exp2.dta", clear
#delimit ;
tempname
/* OLS */
ae_ols_treat pb_ols_treat ase_ols_treat sde_ols_treat srmse_ols_treat zstat_ols_treat pval_ols_treat covr_ols_treat
ae_ols_x1 pb_ols_x1 ase_ols_x1 sde_ols_x1 srmse_ols_x1 zstat_ols_x1 pval_ols_x1 covr_ols_x1
ae_ols_x2 pb_ols_x2 ase_ols_x2 sde_ols_x2 srmse_ols_x2 zstat_ols_x2 pval_ols_x2 covr_ols_x2
ae_ols_d1 pb_ols_d1 ase_ols_d1 sde_ols_d1 srmse_ols_d1 zstat_ols_d1 pval_ols_d1 covr_ols_d1
ae_ols_d2 pb_ols_d2 ase_ols_d2 sde_ols_d2 srmse_ols_d2 zstat_ols_d2 pval_ols_d2 covr_ols_d2
ae_ols_zyvar pb_ols_zyvar ase_ols_zyvar sde_ols_zyvar srmse_ols_zyvar zstat_ols_zyvar pval_ols_zyvar covr_ols_zyvar
/* ivnosel */
ae_ivnosel_treat pb_ivnosel_treat ase_ivnosel_treat sde_ivnosel_treat srmse_ivnosel_treat zstat_ivnosel_treat pval_ivnosel_treat covr_ivnosel_treat
ae_ivnosel_x1 pb_ivnosel_x1 ase_ivnosel_x1 sde_ivnosel_x1 srmse_ivnosel_x1 zstat_ivnosel_x1 pval_ivnosel_x1 covr_ivnosel_x1
ae_ivnosel_x2 pb_ivnosel_x2 ase_ivnosel_x2 sde_ivnosel_x2 srmse_ivnosel_x2 zstat_ivnosel_x2 pval_ivnosel_x2 covr_ivnosel_x2
ae_ivnosel_d1 pb_ivnosel_d1 ase_ivnosel_d1 sde_ivnosel_d1 srmse_ivnosel_d1 zstat_ivnosel_d1 pval_ivnosel_d1 covr_ivnosel_d1
ae_ivnosel_d2 pb_ivnosel_d2 ase_ivnosel_d2 sde_ivnosel_d2 srmse_ivnosel_d2 zstat_ivnosel_d2 pval_ivnosel_d2 covr_ivnosel_d2
ae_ivnosel_zyvar pb_ivnosel_zyvar ase_ivnosel_zyvar sde_ivnosel_zyvar srmse_ivnosel_zyvar zstat_ivnosel_zyvar pval_ivnosel_zyvar covr_ivnosel_zyvar
/* naive2SE */
ae_naive2SE_treat pb_naive2SE_treat ase_naive2SE_treat sde_naive2SE_treat srmse_naive2SE_treat zstat_naive2SE_treat pval_naive2SE_treat covr_naive2SE_treat
ae_naive2SE_x1 pb_naive2SE_x1 ase_naive2SE_x1 sde_naive2SE_x1 srmse_naive2SE_x1 zstat_naive2SE_x1 pval_naive2SE_x1 covr_naive2SE_x1
ae_naive2SE_x2 pb_naive2SE_x2 ase_naive2SE_x2 sde_naive2SE_x2 srmse_naive2SE_x2 zstat_naive2SE_x2 pval_naive2SE_x2 covr_naive2SE_x2
ae_naive2SE_d1 pb_naive2SE_d1 ase_naive2SE_d1 sde_naive2SE_d1 srmse_naive2SE_d1 zstat_naive2SE_d1 pval_naive2SE_d1 covr_naive2SE_d1
ae_naive2SE_d2 pb_naive2SE_d2 ase_naive2SE_d2 sde_naive2SE_d2 srmse_naive2SE_d2 zstat_naive2SE_d2 pval_naive2SE_d2 covr_naive2SE_d2
ae_naive2SE_zyvar pb_naive2SE_zyvar ase_naive2SE_zyvar sde_naive2SE_zyvar srmse_naive2SE_zyvar zstat_naive2SE_zyvar pval_naive2SE_zyvar covr_naive2SE_zyvar
/* Reize2SE */
ae_Reize2SE_treat pb_Reize2SE_treat ase_Reize2SE_treat sde_Reize2SE_treat srmse_Reize2SE_treat zstat_Reize2SE_treat pval_Reize2SE_treat covr_Reize2SE_treat
ae_Reize2SE_x1 pb_Reize2SE_x1 ase_Reize2SE_x1 sde_Reize2SE_x1 srmse_Reize2SE_x1 zstat_Reize2SE_x1 pval_Reize2SE_x1 covr_Reize2SE_x1
ae_Reize2SE_x2 pb_Reize2SE_x2 ase_Reize2SE_x2 sde_Reize2SE_x2 srmse_Reize2SE_x2 zstat_Reize2SE_x2 pval_Reize2SE_x2 covr_Reize2SE_x2
ae_Reize2SE_d1 pb_Reize2SE_d1 ase_Reize2SE_d1 sde_Reize2SE_d1 srmse_Reize2SE_d1 zstat_Reize2SE_d1 pval_Reize2SE_d1 covr_Reize2SE_d1
ae_Reize2SE_d2 pb_Reize2SE_d2 ase_Reize2SE_d2 sde_Reize2SE_d2 srmse_Reize2SE_d2 zstat_Reize2SE_d2 pval_Reize2SE_d2 covr_Reize2SE_d2
ae_Reize2SE_zyvar pb_Reize2SE_zyvar ase_Reize2SE_zyvar sde_Reize2SE_zyvar srmse_Reize2SE_zyvar zstat_Reize2SE_zyvar pval_Reize2SE_zyvar covr_Reize2SE_zyvar
/* Wool3SE */
ae_Wool3SE_treat pb_Wool3SE_treat ase_Wool3SE_treat sde_Wool3SE_treat srmse_Wool3SE_treat zstat_Wool3SE_treat pval_Wool3SE_treat covr_Wool3SE_treat
ae_Wool3SE_x1 pb_Wool3SE_x1 ase_Wool3SE_x1 sde_Wool3SE_x1 srmse_Wool3SE_x1 zstat_Wool3SE_x1 pval_Wool3SE_x1 covr_Wool3SE_x1
ae_Wool3SE_x2 pb_Wool3SE_x2 ase_Wool3SE_x2 sde_Wool3SE_x2 srmse_Wool3SE_x2 zstat_Wool3SE_x2 pval_Wool3SE_x2 covr_Wool3SE_x2
ae_Wool3SE_d1 pb_Wool3SE_d1 ase_Wool3SE_d1 sde_Wool3SE_d1 srmse_Wool3SE_d1 zstat_Wool3SE_d1 pval_Wool3SE_d1 covr_Wool3SE_d1
ae_Wool3SE_d2 pb_Wool3SE_d2 ase_Wool3SE_d2 sde_Wool3SE_d2 srmse_Wool3SE_d2 zstat_Wool3SE_d2 pval_Wool3SE_d2 covr_Wool3SE_d2
ae_Wool3SE_zyvar pb_Wool3SE_zyvar ase_Wool3SE_zyvar sde_Wool3SE_zyvar srmse_Wool3SE_zyvar zstat_Wool3SE_zyvar pval_Wool3SE_zyvar covr_Wool3SE_zyvar;
#delimit cr
{
 tempvar junk
 gen `junk' = .
 /* Generate Montecarlo statistics */
 tempname junk
 gen double `junk' = 0
 mat ols=J(6,8,0)
 mat ivnosel=J(6,8,0)
 mat naive2SE=J(6,8,0)
 mat Reize2SE=J(6,8,0)
 mat Wool3SE=J(6,8,0)
 mat tsls=J(6,8,0)
 /*  OLS */
 #delimit ;
 local param_ols "
  ols_treat ols_x1 ols_x2 ols_d1 ols_d2 ols_zyvar";
 #delimit cr
 local param_mu "y_t y_x1 y_x2 y_d1 y_d2 y_zyvar"
 local i = 1
 foreach x of local param_ols {
  gettoken g param_mu : param_mu 
  tempname sd_ae 
  /* average estimator */
  qui su  `x'
  scalar `ae_`x''=r(mean)
  mat ols[`i',1]= `ae_`x''
  /* standar deviation of the average estimator */
  scalar `sd_ae' = r(sd)
  /* % bias */
  scalar `pb_`x'' = ((`ae_`x''-`mu_`g'')/abs(`mu_`g''))*100
  mat ols[`i',2]=`pb_`x''
  /* average standard error */
  qui su `x'_se
  scalar `ase_`x'' = r(mean)
  mat ols[`i',3]=`ase_`x'' 
  /* Standard deviation of the simulations */
  qui replace `junk' = (`x'-`ae_`x'')^2
  qui su `junk'
  scalar `sde_`x''=sqrt(r(sum)/(`rep'-1))
  mat ols[`i',4]=`sde_`x''
  /* Square-root mean squared error */
  qui replace `junk' = (`x'-`mu_`g'')^2
  qui su `junk'
  scalar `srmse_`x''=sqrt(r(sum)/`rep')
  mat ols[`i',5]= `srmse_`x''
  /* t-stat for H0: bias=0 */
  scalar `zstat_`x'' = ((`ae_`x''-`mu_`g'')/(`sd_ae'/sqrt(`rep')))
  mat ols[`i',6] = `zstat_`x''
  /* p-value */
  scalar `pval_`x'' = 2*(1-normal(abs(`zstat_`x'')))
  mat ols[`i',7] = `pval_`x''
   /* Coverage */
  qui replace `junk'=cond(abs((`x'-`mu_`g'')/`x'_se)<1.96,1,0)
  su `junk'
  scalar `covr_`x'' = r(mean)
  mat ols[`i',8] = `covr_`x''
  local i = `i' + 1
 }
 /*  ivnosel  */
 #delimit ;
 local param_ivnosel "
  ivnosel_treat ivnosel_x1 ivnosel_x2 ivnosel_d1 ivnosel_d2 ivnosel_zyvar";
 #delimit cr
 local param_mu "y_t y_x1 y_x2 y_d1 y_d2 y_zyvar"
 local i = 1
 foreach x of local param_ivnosel {
  gettoken g param_mu : param_mu 
  tempname sd_ae 
  /* average estimator */
  qui su  `x'
  scalar `ae_`x''=r(mean)
  mat ivnosel[`i',1]= `ae_`x''
  /* standar deviation of the average estimator */
  scalar `sd_ae' = r(sd)
  /* % bias */
  scalar `pb_`x'' = ((`ae_`x''-`mu_`g'')/abs(`mu_`g''))*100
  mat ivnosel[`i',2]=`pb_`x''
  /* average standard error */
  qui su `x'_se
  scalar `ase_`x'' = r(mean)
  mat ivnosel[`i',3]=`ase_`x'' 
  /* Standard deviation of the simulations */
  qui replace `junk' = (`x'-`ae_`x'')^2
  qui su `junk'
  scalar `sde_`x''=sqrt(r(sum)/(`rep'-1))
  mat ivnosel[`i',4]=`sde_`x''
  /* Square-root mean squared error */
  qui replace `junk' = (`x'-`mu_`g'')^2
  qui su `junk'
  scalar `srmse_`x''=sqrt(r(sum)/`rep')
  mat ivnosel[`i',5]= `srmse_`x''
  /* t-stat for H0: bias=0 */
  scalar `zstat_`x'' = ((`ae_`x''-`mu_`g'')/(`sd_ae'/sqrt(`rep')))
  mat ivnosel[`i',6] = `zstat_`x''
  /* p-value */
  scalar `pval_`x'' = 2*(1-normal(abs(`zstat_`x'')))
  mat ivnosel[`i',7] = `pval_`x''
   /* Coverage */
  qui replace `junk'=cond(abs((`x'-`mu_`g'')/`x'_se)<1.96,1,0)
  su `junk'
  scalar `covr_`x'' = r(mean)
  mat ivnosel[`i',8] = `covr_`x''
  local i = `i' + 1
 }
 /*  naive2SE  */
 #delimit ;
 local param_naive2SE "
  naive2SE_treat naive2SE_x1 naive2SE_x2 naive2SE_d1 naive2SE_d2 naive2SE_zyvar";
 #delimit cr
 local param_mu "y_t y_x1 y_x2 y_d1 y_d2 y_zyvar"
 local i = 1
 foreach x of local param_naive2SE {
  gettoken g param_mu : param_mu 
  tempname sd_ae 
  /* average estimator */
  qui su  `x'
  scalar `ae_`x''=r(mean)
  mat naive2SE[`i',1]= `ae_`x''
  /* standar deviation of the average estimator */
  scalar `sd_ae' = r(sd)
  /* % bias */
  scalar `pb_`x'' = ((`ae_`x''-`mu_`g'')/abs(`mu_`g''))*100
  mat naive2SE[`i',2]=`pb_`x''
  /* average standard error */
  qui su `x'_se
  scalar `ase_`x'' = r(mean)
  mat naive2SE[`i',3]=`ase_`x'' 
  /* Standard deviation of the simulations */
  qui replace `junk' = (`x'-`ae_`x'')^2
  qui su `junk'
  scalar `sde_`x''=sqrt(r(sum)/(`rep'-1))
  mat naive2SE[`i',4]=`sde_`x''
  /* Square-root mean squared error */
  qui replace `junk' = (`x'-`mu_`g'')^2
  qui su `junk'
  scalar `srmse_`x''=sqrt(r(sum)/`rep')
  mat naive2SE[`i',5]= `srmse_`x''
  /* t-stat for H0: bias=0 */
  scalar `zstat_`x'' = ((`ae_`x''-`mu_`g'')/(`sd_ae'/sqrt(`rep')))
  mat naive2SE[`i',6] = `zstat_`x''
  /* p-value */
  scalar `pval_`x'' = 2*(1-normal(abs(`zstat_`x'')))
  mat naive2SE[`i',7] = `pval_`x''
   /* Coverage */
  qui replace `junk'=cond(abs((`x'-`mu_`g'')/`x'_se)<1.96,1,0)
  su `junk'
  scalar `covr_`x'' = r(mean)
  mat naive2SE[`i',8] = `covr_`x''
  local i = `i' + 1
 }
 /*  Reize2SE  */
 #delimit ;
 local param_Reize2SE "
  Reize2SE_treat Reize2SE_x1 Reize2SE_x2 Reize2SE_d1 Reize2SE_d2 Reize2SE_zyvar";
 #delimit cr
 local param_mu "y_t y_x1 y_x2 y_d1 y_d2 y_zyvar"
 local i = 1
 foreach x of local param_Reize2SE {
  gettoken g param_mu : param_mu 
  tempname sd_ae 
  /* average estimator */
  qui su  `x'
  scalar `ae_`x''=r(mean)
  mat Reize2SE[`i',1]= `ae_`x''
  /* standar deviation of the average estimator */
  scalar `sd_ae' = r(sd)
  /* % bias */
  scalar `pb_`x'' = ((`ae_`x''-`mu_`g'')/abs(`mu_`g''))*100
  mat Reize2SE[`i',2]=`pb_`x''
  /* average standard error */
  qui su `x'_se
  scalar `ase_`x'' = r(mean)
  mat Reize2SE[`i',3]=`ase_`x'' 
  /* Standard deviation of the simulations */
  qui replace `junk' = (`x'-`ae_`x'')^2
  qui su `junk'
  scalar `sde_`x''=sqrt(r(sum)/(`rep'-1))
  mat Reize2SE[`i',4]=`sde_`x''
  /* Square-root mean squared error */
  qui replace `junk' = (`x'-`mu_`g'')^2
  qui su `junk'
  scalar `srmse_`x''=sqrt(r(sum)/`rep')
  mat Reize2SE[`i',5]= `srmse_`x''
  /* t-stat for H0: bias=0 */
  scalar `zstat_`x'' = ((`ae_`x''-`mu_`g'')/(`sd_ae'/sqrt(`rep')))
  mat Reize2SE[`i',6] = `zstat_`x''
  /* p-value */
  scalar `pval_`x'' = 2*(1-normal(abs(`zstat_`x'')))
  mat Reize2SE[`i',7] = `pval_`x''
   /* Coverage */
  qui replace `junk'=cond(abs((`x'-`mu_`g'')/`x'_se)<1.96,1,0)
  su `junk'
  scalar `covr_`x'' = r(mean)
  mat Reize2SE[`i',8] = `covr_`x''
  local i = `i' + 1
 }
 /*  Wool3SE  */
 #delimit ;
 local param_Wool3SE "
  Wool3SE_treat Wool3SE_x1 Wool3SE_x2 Wool3SE_d1 Wool3SE_d2 Wool3SE_zyvar";
 #delimit cr
 local param_mu "y_t y_x1 y_x2 y_d1 y_d2 y_zyvar"
 local i = 1
 foreach x of local param_Wool3SE {
  gettoken g param_mu : param_mu 
  tempname sd_ae 
  /* average estimator */
  qui su  `x'
  scalar `ae_`x''=r(mean)
  mat Wool3SE[`i',1]= `ae_`x''
  /* standar deviation of the average estimator */
  scalar `sd_ae' = r(sd)
  /* % bias */
  scalar `pb_`x'' = ((`ae_`x''-`mu_`g'')/abs(`mu_`g''))*100
  mat Wool3SE[`i',2]=`pb_`x''
  /* average standard error */
  qui su `x'_se
  scalar `ase_`x'' = r(mean)
  mat Wool3SE[`i',3]=`ase_`x'' 
  /* Standard deviation of the simulations */
  qui replace `junk' = (`x'-`ae_`x'')^2
  qui su `junk'
  scalar `sde_`x''=sqrt(r(sum)/(`rep'-1))
  mat Wool3SE[`i',4]=`sde_`x''
  /* Square-root mean squared error */
  qui replace `junk' = (`x'-`mu_`g'')^2
  qui su `junk'
  scalar `srmse_`x''=sqrt(r(sum)/`rep')
  mat Wool3SE[`i',5]= `srmse_`x''
  /* t-stat for H0: bias=0 */
  scalar `zstat_`x'' = ((`ae_`x''-`mu_`g'')/(`sd_ae'/sqrt(`rep')))
  mat Wool3SE[`i',6] = `zstat_`x''
  /* p-value */
  scalar `pval_`x'' = 2*(1-normal(abs(`zstat_`x'')))
  mat Wool3SE[`i',7] = `pval_`x''
   /* Coverage */
  qui replace `junk'=cond(abs((`x'-`mu_`g'')/`x'_se)<1.96,1,0)
  su `junk'
  scalar `covr_`x'' = r(mean)
  mat Wool3SE[`i',8] = `covr_`x''
  local i = `i' + 1
 }
 /* Display output program */
 mat M = J(1,5,.)
 qui su psel
 mat M[1,1] = r(mean)
 qui su nsel
 mat M[1,2] = r(mean)
 qui su ptreat
 mat M[1,3] = r(mean)
 qui su ntreat
 mat M[1,4] = r(mean)
 mat M[1,5] = nfail[_N]
 /* DISPLAY */
 mata:
 function Display()
 {
  /* Declarations */
  real matrix ols, ivnosel, naive2SE, Reize2SE, Wool3SE
  string matrix E, V, D
  scalar i,j
  /* parse matrices */
  OLS = st_matrix("ols")
  IVNOSEL = st_matrix("ivnosel")
  NAIVE2SE= st_matrix("naive2SE")
  REIZE2SE = st_matrix("Reize2SE")
  Wool3SE = st_matrix("Wool3SE") 
  M = st_matrix("M")
  E = ("Ordinary Least Squares","IV no selection", "Naive Two Stage Estimator", "Reize Two Stage Estimator", "Wooldridge Three Stage Estimator")
  D =("Mutivariate Normal")
  V = ("treat", "x1","x2","d1","d2", "zyvar")
  PP = st_matrix("PP")
 /* Display */
  printf("\n")
  printf("{txt} Monte Carlo Simulation\n")
  printf("\n")
  printf("{txt} Total Number of obs    = {res}%7.0g \n",PP[1,1])
  printf("{txt} Number of repetitions  = {res}%7.0g \n",PP[1,2])
  printf("{txt} mu_sd_y                = {res}%7.0g \n",PP[1,3])
  printf("{txt} mu_sd_t                = {res}%7.0g \n",PP[1,4])
  printf("{txt} mu_sd_s                = {res}%7.0g \n",PP[1,5])
  printf("{txt} mu_rho_yt              = {res}%7.0g \n",PP[1,6])
  printf("{txt} mu_rho_ys              = {res}%7.0g \n",PP[1,7])
  printf("{txt} mu_rho_ts              = {res}%7.0g \n",PP[1,8])
  printf("{txt} mu_t_x1                = {res}%7.0g \n",PP[1,9])
  printf("{txt} mu_t_x2                = {res}%7.0g \n",PP[1,10])
  printf("{txt} mu_t_d1                = {res}%7.0g \n",PP[1,11])
  printf("{txt} mu_t_d1                = {res}%7.0g \n",PP[1,12])
  printf("{txt} mu_t_ztreat            = {res}%7.0g \n",PP[1,13])
  printf("{txt} mu_t_cons              = {res}%7.0g \n",PP[1,14])
  printf("{txt} mu_y_treat             = {res}%7.0g \n",PP[1,15])
  printf("{txt} mu_y_x1                = {res}%7.0g \n",PP[1,16])
  printf("{txt} mu_y_x2                = {res}%7.0g \n",PP[1,17])
  printf("{txt} mu_y_d1                = {res}%7.0g \n",PP[1,18])
  printf("{txt} mu_y_d2                = {res}%7.0g \n",PP[1,19])
  printf("{txt} mu_y_yvar              = {res}%7.0g \n",PP[1,20])
  printf("{txt} mu_y_cons              = {res}%7.0g \n",PP[1,21])
  printf("{txt} mu_s_treat             = {res}%7.0g \n",PP[1,22])
  printf("{txt} mu_s_x1                = {res}%7.0g \n",PP[1,23])
  printf("{txt} mu_s_x2                = {res}%7.0g \n",PP[1,24])
  printf("{txt} mu_s_d1                = {res}%7.0g \n",PP[1,25])
  printf("{txt} mu_s_d1                = {res}%7.0g \n",PP[1,26])
  printf("{txt} mu_s_zsel              = {res}%7.0g \n",PP[1,27])
  printf("{txt} mu_s_cons              = {res}%7.0g \n",PP[1,28])
  printf("{txt} noise/sig ratio (y)    = {res}%7.0g \n",PP[1,29])
  printf("{txt} noise/sig ratio (t)    = {res}%7.0g \n",PP[1,30])
  printf("{txt} noise/sig ratio (s)    = {res}%7.0g \n",PP[1,31])
  printf("{txt} seed                   = {res}%7.0g \n",PP[1,32])
  printf("{txt} Mean Pr of selection   = {res}%7.0g \n",M[1,1])
  printf("{txt} Mean number  sel. obs  = {res}%7.0g \n",M[1,2])
  printf("{txt} Mean Pr of treatment   = {res}%7.0g \n",M[1,3])
  printf("{txt} Mean number treat obs  = {res}%7.0g \n",M[1,4])
  printf("{txt} # fail to converge     = {res}%7.0g \n",M[1,5])
  printf("{txt} Distribution of errors = {res}%7s\n",D[1,1])
  printf("\n")
  printf("{txt} {hline 75}\n")
  printf("{txt}{space 12}{ralign 7:AE}{space 1}{ralign 7:PB}{space 1}{ralign 7:ASE}{space 1}{ralign 7:SD}{space 1}{ralign 7:RMSE}{space 1}{ralign 7:|z|}{space 1}{ralign 7:p-val}{space 1}{ralign 7:Covrg}\n")
  printf("{txt} {hline 75}\n")
  for (i=1;i<=cols(E);i++) {
   printf("{text} %s \n",E[1,i])
    for (j=1;j<=cols(V);j++) {
     if (E[1,i]=="Ordinary Least Squares") {
      printf(" {txt}%-10s{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.3f{space 1}{res}%7.0g\n",V[1,j],OLS[j,1],OLS[j,2],OLS[j,3],OLS[j,4],OLS[j,5],OLS[j,6],OLS[j,7],OLS[j,8])
     }
    }
    for (j=1;j<=cols(V);j++) {
     if (E[1,i]=="IV no selection") {
      printf(" {txt}%-10s{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.3f{space 1}{res}%7.0g\n",V[1,j],IVNOSEL[j,1],IVNOSEL[j,2],IVNOSEL[j,3],IVNOSEL[j,4],IVNOSEL[j,5],IVNOSEL[j,6],IVNOSEL[j,7],IVNOSEL[j,8])
     }
    }
    for (j=1;j<=cols(V);j++) {
     if (E[1,i]=="Naive Two Stage Estimator") {
      printf(" {txt}%-10s{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.3f{space 1}{res}%7.0g\n",V[1,j],NAIVE2SE[j,1],NAIVE2SE[j,2],NAIVE2SE[j,3],NAIVE2SE[j,4],NAIVE2SE[j,5],NAIVE2SE[j,6],NAIVE2SE[j,7],NAIVE2SE[j,8])
     }
    }
	for (j=1;j<=cols(V);j++) {
     if (E[1,i]=="Wooldridge Three Stage Estimator") {
      printf(" {txt}%-10s{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.3f{space 1}{res}%7.0g\n",V[1,j],Wool3SE[j,1],Wool3SE[j,2],Wool3SE[j,3],Wool3SE[j,4],Wool3SE[j,5],Wool3SE[j,6],Wool3SE[j,7],Wool3SE[j,8])
     }
    }
    for (j=1;j<=cols(V);j++) {
     if (E[1,i]=="Reize Two Stage Estimator") {
      printf(" {txt}%-10s{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.0g{space 1}{res}%7.3f{space 1}{res}%7.0g\n",V[1,j],REIZE2SE[j,1],REIZE2SE[j,2],REIZE2SE[j,3],REIZE2SE[j,4],REIZE2SE[j,5],REIZE2SE[j,6],REIZE2SE[j,7],REIZE2SE[j,8])
     }
    }
   }
  printf("{txt} {hline 77}\n")
 }
 end
}
log using "$locallogpath/Montecarlo_exp2.log", replace
mata: Display()
log close
mata: mata clear



